@@ -254,12 +254,12 @@ totem_action_exit (Totem *totem)
 	if (totem->gc)
 		g_object_unref (G_OBJECT (totem->gc));
 
-	if (totem->win)
-		gtk_widget_destroy (GTK_WIDGET (totem->win));
-
 	if (totem->fs)
 		g_object_unref (totem->fs);
 
+	if (totem->win)
+		gtk_widget_destroy (GTK_WIDGET (totem->win));
+
 	g_object_unref (totem);
 
 	gnome_vfs_shutdown ();
@@ -481,15 +481,40 @@ totem_action_load_media (Totem *totem, TotemDiscMediaType type, const char *devi
 	char **mrls;
 	char *msg;
 	gboolean retval;
+	BaconVideoWidgetCanPlayStatus status;
 
-	if (bacon_video_widget_can_play (totem->bvw, type) == FALSE) {
-		if (type == MEDIA_TYPE_DVD || type == MEDIA_TYPE_VCD)
-			msg = g_strdup_printf(_("Totem cannot play this type of media (%s) because it does not have the appropriate plugins to be able to read from the disc."), _(totem_cd_get_human_readable_name (type)));
-		else
-			msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because you do not have the appropriate plugins to handle it."), _(totem_cd_get_human_readable_name (type)));
-		totem_interface_error_with_link (msg, _("Please install the necessary plugins and restart Totem to be able to play this media."),
-				"http://www.gnome.org/projects/totem/#codecs", _("More information about media plugins"),
-				GTK_WINDOW (totem->win), totem);
+	status = bacon_video_widget_can_play (totem->bvw, type);
+
+	if (status != BVW_CAN_PLAY_SUCCESS) {
+		const char *link, *link_text, *secondary;
+
+		switch (status) {
+		case BVW_CAN_PLAY_MISSING_CHANNELS:
+			g_assert (type == MEDIA_TYPE_DVB);
+			link = "http://www.gnome.org/projects/totem/#dvb";
+			link_text = _("More information about watching TV");
+			msg = g_strdup (_("Totem is missing a channels listing to be able to tune the receiver."));
+			secondary = _("Please follow the instructions provided in the link to create a channels listing.");
+			break;
+		case BVW_CAN_PLAY_MISSING_PLUGINS:
+			link = "http://www.gnome.org/projects/totem/#codecs";
+			link_text = _("More information about media plugins");
+			secondary = _("Please install the necessary plugins and restart Totem to be able to play this media.");
+			if (type == MEDIA_TYPE_DVD || type == MEDIA_TYPE_VCD)
+				msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because it does not have the appropriate plugins to be able to read from the disc."), _(totem_cd_get_human_readable_name (type)));
+			else
+				msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because you do not have the appropriate plugins to handle it."), _(totem_cd_get_human_readable_name (type)));
+			break;
+		case BVW_CAN_PLAY_UNSUPPORTED:
+			msg = g_strdup_printf(_("Totem cannot play this type of media (%s) because it is not supported."), _(totem_cd_get_human_readable_name (type)));
+			totem_action_error (msg, _("Please insert another disc to play back."), totem);
+			g_free (msg);
+			return FALSE;
+		default:
+			g_assert_not_reached ();
+		}
+
+		totem_interface_error_with_link (msg, secondary, link, link_text, GTK_WINDOW (totem->win), totem);
 		g_free (msg);
 		return FALSE;
 	}
@@ -537,14 +562,7 @@ totem_action_load_media_device (Totem *totem, const char *device)
 			break;
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
-			{
-				const char *filenames[2];
-
-				filenames[0] = url;
-				filenames[1] = NULL;
-
-				retval = totem_action_open_files (totem, (char **) filenames);
-			}
+			retval = totem_action_load_media (totem, type, device_path);
 			break;
 		case MEDIA_TYPE_CDDA:
 			totem_action_error (_("Totem does not support playback of Audio CDs"),
@@ -649,11 +667,14 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		return FALSE;
 
 	if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
-		if (totem->controls_visibility != TOTEM_CONTROLS_UNDEFINED)
+		if (totem->controls_visibility != TOTEM_CONTROLS_UNDEFINED) {
+			g_message ("vis is %d", totem->controls_visibility);
 			totem_action_save_size (totem);
+		}
 		totem_fullscreen_set_fullscreen (totem->fs, TRUE);
 
 		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
+		g_message ("setting to fullscreen now");
 		show_controls (totem, FALSE);
 		totem_action_set_sensitivity ("fullscreen", FALSE);
 	} else {
@@ -664,10 +685,13 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		action = gtk_action_group_get_action (totem->main_action_group,
 				"show-controls");
 
-		if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
+		if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
 			totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
-		else
+			g_message ("vis is %d (is active)", totem->controls_visibility);
+		} else {
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
+			g_message ("vis is %d (is not active)", totem->controls_visibility);
+		}
 
 		show_controls (totem, TRUE);
 		totem_action_set_sensitivity ("fullscreen", TRUE);
@@ -678,12 +702,6 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 	return FALSE;
 }
 
-void
-fs_exit1_activate_cb (GtkButton *button, Totem *totem)
-{
-	totem_action_fullscreen_toggle (totem);
-}
-
 void
 totem_action_fullscreen_toggle (Totem *totem)
 {
@@ -702,6 +720,12 @@ totem_action_fullscreen (Totem *totem, gboolean state)
 	totem_action_fullscreen_toggle (totem);
 }
 
+void
+fs_exit1_activate_cb (GtkButton *button, Totem *totem)
+{
+	totem_action_fullscreen (totem, FALSE);
+}
+
 void
 totem_action_open (Totem *totem)
 {
@@ -1181,14 +1205,14 @@ totem_action_volume_relative (Totem *totem, double off_pct)
 
 void
 totem_action_toggle_aspect_ratio (Totem *totem)
-{		
+{
 	GtkAction *action;
 	int tmp;
 
 	tmp = totem_action_get_aspect_ratio (totem);
 	tmp++;
-	if (tmp > 4)
-		tmp = 0;
+	if (tmp > BVW_RATIO_DVB)
+		tmp = BVW_RATIO_AUTO;
 
 	action = gtk_action_group_get_action (totem->main_action_group, "aspect-ratio-auto");
 	gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), tmp);
@@ -1230,26 +1254,17 @@ static gboolean
 totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 		int drop_type, gboolean empty_pl)
 {
-	GList *list, *p, *file_list;
+	char **list;
+	guint i;
+	GList *p, *file_list;
 	gboolean cleared = FALSE;
 
-	list = gnome_vfs_uri_list_parse ((const char *)data->data);
-
-	if (list == NULL)
-		return FALSE;
-
-	p = list;
+	list = g_uri_list_extract_uris ((const char *)data->data);
 	file_list = NULL;
 
-	while (p != NULL)
-	{
-		file_list = g_list_prepend (file_list, 
-				gnome_vfs_uri_to_string
-				((const GnomeVFSURI*)(p->data), 0));
-		p = p->next;
-	}
+	for (i = 0; list[i] != NULL; ++i)
+		file_list = g_list_prepend (file_list, list[i]);
 
-	gnome_vfs_uri_list_free (list);
 	if (file_list == NULL)
 		return FALSE;
 
@@ -1287,7 +1302,6 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 		/* Super _NETSCAPE_URL trick */
 		if (drop_type == 1)
 		{
-			g_free (p->data);
 			p = p->next;
 			if (p != NULL) {
 				if (g_str_has_prefix (p->data, "file:") != FALSE)
@@ -1300,9 +1314,9 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 		totem_playlist_add_mrl (totem->playlist, filename, title);
 
 		g_free (filename);
-		g_free (p->data);
 	}
 
+	g_strfreev (list);
 	g_list_free (file_list);
 	gdk_window_set_cursor (totem->win->window, NULL);
 
@@ -2036,13 +2050,17 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_MUTE:
 		totem_action_volume_relative (totem, -1.0);
 		break;
+	case TOTEM_REMOTE_COMMAND_TOGGLE_ASPECT:
+		totem_action_toggle_aspect_ratio (totem);
+		break;
 	default:
 		handled = FALSE;
 		break;
 	}
 
-	if (handled != FALSE &&
-			gtk_window_is_active (GTK_WINDOW (totem->win))) {
+	if (handled != FALSE
+			&& gtk_window_is_active (GTK_WINDOW (totem->win))
+			&& totem_fullscreen_is_fullscreen (totem->fs) != FALSE) {
 		totem_fullscreen_motion_notify (NULL, NULL, totem->fs);
 	}
 }
@@ -2224,7 +2242,8 @@ totem_is_seekable (Totem *totem)
 static void
 on_mouse_click_fullscreen (GtkWidget *widget, Totem *totem)
 {
-	totem_fullscreen_motion_notify (NULL, NULL, totem->fs);
+	if (totem_fullscreen_is_fullscreen (totem->fs) != FALSE)
+		totem_fullscreen_motion_notify (NULL, NULL, totem->fs);
 }
 
 static gboolean
@@ -2539,7 +2558,8 @@ totem_action_handle_scroll (Totem *totem, GdkScrollDirection direction)
 {
 	gboolean retval = TRUE;
 
-	totem_fullscreen_motion_notify (NULL, NULL, totem->fs);
+	if (totem_fullscreen_is_fullscreen (totem->fs) != FALSE)
+		totem_fullscreen_motion_notify (NULL, NULL, totem->fs);
 
 	switch (direction) {
 	case GDK_SCROLL_UP:
@@ -2952,7 +2972,6 @@ video_widget_create (Totem *totem)
 	}
 
 	totem_preferences_tvout_setup (totem);
-	totem_preferences_visuals_setup (totem);
 	totem_action_zoom (totem, ZOOM_RESET);
 
 	g_signal_connect_after (G_OBJECT (totem->bvw),
@@ -3026,6 +3045,8 @@ video_widget_create (Totem *totem)
 	gtk_widget_realize (GTK_WIDGET (totem->bvw));
 	gtk_widget_show (GTK_WIDGET (totem->bvw));
 
+	totem_preferences_visuals_setup (totem);
+
 	bacon_video_widget_set_volume (totem->bvw,
 			((double) gconf_client_get_int (totem->gc,
 				GCONF_PREFIX"/volume", NULL)) / 100.0);
@@ -3170,7 +3191,6 @@ main (int argc, char **argv)
 	gtk_range_set_adjustment (GTK_RANGE (totem->fs->seek), totem->seekadj);
 
 	totem_session_setup (totem, argv);
-	totem_setup_recent (totem);
 	totem_setup_file_monitoring (totem);
 	totem_setup_file_filters ();
 	totem_setup_play_disc (totem);
@@ -3195,16 +3215,16 @@ main (int argc, char **argv)
 	totem_fullscreen_set_video_widget (totem->fs, totem->bvw);
 
 	if (optionstate.fullscreen != FALSE) {
-		totem_action_fullscreen (totem, TRUE);
-		long_action ();
 		gtk_widget_show (totem->win);
 		gdk_flush ();
+		totem_action_fullscreen (totem, TRUE);
 	}
-	long_action ();
 
 	/* The prefs after the video widget is connected */
 	totem_setup_preferences (totem);
 
+	totem_setup_recent (totem);
+
 	/* Command-line handling */
 	totem_options_process_late (totem, &optionstate);
 
