@@ -66,7 +66,8 @@ typedef enum {
 	TOTEM_REMOTE_COMMAND_ZOOM_DOWN,
 	TOTEM_REMOTE_COMMAND_EJECT,
 	TOTEM_REMOTE_COMMAND_PLAY_DVD,
-	TOTEM_REMOTE_COMMAND_MUTE
+	TOTEM_REMOTE_COMMAND_MUTE,
+	TOTEM_REMOTE_COMMAND_TOGGLE_ASPECT
 } TotemRemoteCommand;
 
 typedef enum {
